#!/usr/bin/env python
#
# Copyright 2013 Splunk, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"): you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sys

from P4 import P4, P4Exception
from splunklib.modularinput import *


try:
    import xml.etree.cElementTree as ET
except ImportError:
    import xml.etree.ElementTree as ET

class MyScript(Script):
    """All modular inputs should inherit from the abstract base class Script
    from splunklib.modularinput.script.
    They must override the get_scheme and stream_events functions, and,
    if the scheme returned by get_scheme has Scheme.use_external_validation
    set to True, the validate_input function.
    """
    def get_scheme(self):
        """When Splunk starts, it looks for all the modular inputs defined by
        its configuration, and tries to run them with the argument --scheme.
        Splunkd expects the modular inputs to print a description of the
        input in XML on stdout. The modular input framework takes care of all
        the details of formatting XML and printing it. The user need only
        override get_scheme and return a new Scheme object.

        :return: scheme, a Scheme object
        """
        # "p4_journal" is the name Splunk will display to users for this input.
        scheme = Scheme("P4 Journal")

        scheme.description = "Streams Perforce journal records."
        # If you set external validation to True, without overriding validate_input,
        # the script will accept anything as valid. Generally you only need external
        # validation if there are relationships you must maintain among the
        # parameters, such as requiring min to be less than max in this example,
        # or you need to check that some resource is reachable or valid.
        # Otherwise, Splunk lets you specify a validation string for each argument
        # and will run validation internally using that string.
        scheme.use_external_validation = True
        scheme.use_single_instance = True

        p4port = Argument("p4port")
        p4port.data_type = Argument.data_type_string
        p4port.description = "Perforce server port e.g. 'localhost:1666'"
        p4port.required_on_create = True
        scheme.add_argument(p4port)
        
        p4user = Argument("p4user")
        p4user.data_type = Argument.data_type_string
        p4user.description = "Username to authenticate against the Perforce server"
        p4user.required_on_create = True
        scheme.add_argument(p4user)
        
        p4pass = Argument("p4pass")
        p4pass.data_type = Argument.data_type_string
        p4pass.description = "Password or Ticket to authenticate against the Perforce server"
        p4pass.required_on_create = False
        scheme.add_argument(p4pass)

        return scheme

    def validate_input(self, validation_definition):
        """In this example we are using external validation to verify that min is
        less than max. If validate_input does not raise an Exception, the input is
        assumed to be valid. Otherwise it prints the exception as an error message
        when telling splunkd that the configuration is invalid.

        When using external validation, after splunkd calls the modular input with
        --scheme to get a scheme, it calls it again with --validate-arguments for
        each instance of the modular input in its configuration files, feeding XML
        on stdin to the modular input to do validation. It is called the same way
        whenever a modular input's configuration is edited.

        :param validation_definition: a ValidationDefinition object
        """
        
        p4port = validation_definition.parameters["p4port"]
        p4user = validation_definition.parameters["p4user"]
        p4pass = validation_definition.parameters["p4pass"]
        
        p4 = self.connection(p4port, p4user, p4pass)
        try:
            p4.run_info()
        finally:
            p4.disconnect() 
                  
                           
    def stream_events(self, inputs, ew):
        """This function handles all the action: splunk calls this modular input
        without arguments, streams XML describing the inputs to stdin, and waits
        for XML on stdout describing events.

        If you set use_single_instance to True on the scheme in get_scheme, it
        will pass all the instances of this input to a single instance of this
        script.

        :param inputs: an InputDefinition object
        :param ew: an EventWriter object
        """
        # Go through each input for this modular input
        for input_name, item in inputs.inputs.iteritems():          
            
            p4 = self.connection(item["p4port"], item["p4user"], item["p4pass"])
            try:
                p4.run_info()
            finally:
                p4.disconnect() 
            
            # Create an Event object, and set its data fields
            event = Event()
            event.stanza = input_name
            event.data = "user=" + p4.user

            # Tell the EventWriter to write this event
            ew.write_event(event)


    def connection(self, p4port, p4user, p4pass):
        p4 = P4()
        p4.exception_level = 1
        p4.prog = "p4-splunk"
        p4.version = "1.0"
            
        p4.port = p4port
        p4.user = p4user
        p4.password = p4pass
            
        try:
            p4.connect()
        except P4Exception:
            raise ValueError("Unable to connect to Perforce" + p4.errors)
                
        try:
            p4pass = validation_definition.parameters["p4pass"]
            if p4pass:
                p4.password = p4pass  
            p4.run_login()
        except P4Exception:
            raise ValueError("Unable to login to Perforce" + p4.errors)
                
        return p4


if __name__ == "__main__":
    sys.exit(MyScript().run(sys.argv))
